# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Helper functions to send mails
#
# Authors:
# Andre Heinecke <aheinecke@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import smtplib
import email
import email.Utils

#TODO get more information in the mails.
#TODO write serious templates
#TODO Unicode Support
TEMPLATE_FINISHED="""From: %(sender)s
To: %(reciever)s
Subject: mapWOC: %(scanname)s completed.
MIME-Version: 1.0
Content-type: text/plain

%(scanname)s completed successfully.

"""

TEMPLATE_FINISHED_DE="""From: %(sender)s
To: %(reciever)s
Subject: mapWOC: %(scanname)s erfolgreich beendet.
MIME-Version: 1.0
Content-type: text/plain

%(scanname)s wurde erfolgreich beendet.

"""

TEMPLATE_ERROR="""From: %(sender)s
To: %(reciever)s
Subject: mapWOC: Error %(errornr)d
MIME-Version: 1.0
Content-type: text/plain

Error: %(errornr)d
Errormessage: %(errormessage)s

"""

TEMPLATE_ERROR_DE="""From: %(sender)s
To: %(reciever)s
Subject: mapWOC: Fehler %(errornr)d
MIME-Version: 1.0
Content-type: text/plain

Fehler: %(errornr)d aufgetreten.
Fehlermeldung: %(errormessage)s

"""

TEMPLATE_FOUND="""From: %(sender)s
To: %(reciever)s
Subject: mapWOC: %(highest_severity)s (%(result_types)s) found in scan %(scan)s.
MIME-Version: 1.0
Content-type: text/plain

Summary:
    Scanned URL: %(url)s
    Time: %(date)s
    Honey Client: %(hc)s
    Result Type: %(result_types)s
    Highest Severity: %(highest_severity)s

For additional details please see: %(gui_url)s/result/view_visit/%(visit_id)s
"""

TEMPLATE_FOUND_DE="""From: %(sender)s
To: %(reciever)s
Subject: mapWOC: %(highest_severity)s (%(result_types)s) in Scan %(scan)s gefunden.
MIME-Version: 1.0
Content-type: text/plain

Zusammenfassung:
    Gescannte URL: %(url)s
    Zeitpunkt: %(date)s
    Honey Client: %(hc)s
    Typ: %(result_types)s
    Maximaler Schweregrad: %(highest_severity)s

Weitere Details unter: %(gui_url)s/result/view_visit/%(visit_id)s
"""

class MailDispatcher(object):
    """ Helper class to send out mails. """

    def __init__(self, smtp_host, smtp_port, admin_mail, sender, gui_url = None):
        if(smtp_port == None or
           smtp_host == None or
           admin_mail== None or
           sender == None):
            self.disabled = True
        else:
            self.disabled = False
        self.smtp_host = smtp_host
        self.smtp_port = smtp_port
        self.vars = {"reciever": admin_mail,
                     "sender" : sender }
        if gui_url:
            self.vars["gui_url"] = gui_url
        else:
            self.vars["gui_url"] = "https://localhost"

    def send_severity_found(self, **kw):
        """
        Send the template_found template

        @param kw: Template variables. May not be Unicode
        @param language: Language of the template to send
        """
        vars = self.vars.copy()
        vars.update(kw)
        if "language" in kw and kw["language"] == "de":
            self.send_mail(TEMPLATE_FOUND_DE % vars)
        else:
            self.send_mail(TEMPLATE_FOUND % vars)

    def send_scan_finished(self, **kw):
        """
        Send the scan_finished template.

        @param kw: Template variables. May not be Unicode
        @param language: Language of the template to send
        """
        vars = self.vars.copy()
        vars.update(kw)
        if "language" in kw and kw["language"] == "de":
            self.send_mail(TEMPLATE_FINISHED_DE % vars)
        else:
            self.send_mail(TEMPLATE_FINISHED % vars)

    def send_error(self, **kw):
        """
        Send the error template.

        @param kw: Template variables. May not be Unicode
        @param language: Language of the template to send
        """
        vars = self.vars.copy()
        vars.update(kw)
        if "language" in kw and kw["language"] == "de":
            self.send_mail(TEMPLATE_ERROR_DE % vars)
        else:
            self.send_mail(TEMPLATE_ERROR % vars)

    def send_mail(self, raw_message):
        """
        Send a mail.

        @param raw_message: A correctly formatted mail.
        """
        if self.disabled: return
        msg = email.message_from_string(raw_message)
        sender = email.Utils.parseaddr(msg["From"])[1]
        recipients = [addr[1] for addr in
                        email.Utils.getaddresses(msg.get_all("To", [])
                                       + msg.get_all("Cc", []))]
        server = smtplib.SMTP(self.smtp_host, self.smtp_port)
        server.sendmail(sender, recipients, raw_message)
        server.quit()
